/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include <project.h>

#define LSBperGee 16384.0      //from the Register map, there are 16384 LSB's per gee in +/- 2 gee mode

void MPU_ByteWrite(uint8_t reg, uint8_t byte);      //ByteWrite prototype
void runLED();

uint8_t xtopByte;           //we'll use these to hold and process the data from the MPU
uint8_t xbottomByte;
uint16_t xcombinedData;

uint8_t ytopByte;
uint8_t ybottomByte;
uint16_t ycombinedData;

float xgeeForce;
float ygeeForce;

int LED_MODE = 0;       //this is the state variable for our LED


int main()
{
    CyGlobalIntEnable;          //Enable Global Interrupts

    I2C_MASTER_1_Start();       //Start the I2C Master
    
    /*
    The steps for writing a single byte over I2C is as follows:
    1- Master sends a start condition
    2- Master sends write bit + I2C Address
    3- Peripheral sends ACK
    4-Master sends data byte
    5- Peripheral sends ACK
    6- Master sends stop condition
    
    Check the datasheet for the exact procedure for a given chip.
    
    To write a register on the MPU 6050 you must:
    1- send start condition
    2- write + I2C address
    3- send register address
    4- send databyte
    5- send stop condition
    */
    
    
    I2C_MASTER_1_MasterSendStart(0x68, 0);      //starts an I2C write. The address of the MPU is 0x68
    I2C_MASTER_1_MasterWriteByte(0x1A);         //writes the address of the target register. 1A is the CONFIG register
    I2C_MASTER_1_MasterWriteByte(0x00);         //writes sets the frame sync to disabled and the DLPF to a bandwidth of 260 Hz
    I2C_MASTER_1_MasterSendStop();              //stops an I2C transaction.
    
    /*Since it's the same procedure for every single-register write on this chip, this procedure has
      been wrapped into a helper function, void MPU_ByteWrite(uint8_t I2CAddress, uint8_t dataByte).
    */
    
    
    MPU_ByteWrite(0x6B, 0x00);  //takes the MPU out of sleep mode
    MPU_ByteWrite(0x1C, 0x00);  //sets the accelerometer to +/- 2 gees range
    MPU_ByteWrite(0x23, 0x00);  //disables FIFO buffer and DLPF
    
    
    for(;;)
    {
        
    /* The accelerometer data for the three axes is held in sequential registers
       starting at 0x3B. To read the data, we can do what's called a sequential read
       (also called burst read) of the registers.
        The procedure for reading is:
        1- Master sends start condition
        2- Master sends write + I2C address
        3- Master writes the starting address of the read
        4- Master sends restart condition (The same as a start, but without first sending a stop condition)
        5- Master sends read + I2C address
        A:
        6- Peripheral sends a data byte
        If you're going to read more bytes:
            7- Master sends acknowledgement (ACK)
            GOTO A
        Otherwise,
        8- Master sends nonacknowledgement (NAK)
        9- Master sends stop condition
        
        This MPU will automatically move to the next register address after each read,
        so burst reading the data works well.
        
        
        The data in the MPU is stored as 8-bit values to make two halves of a 16-bit
        signed integer. PSoC assumed all bytes read are 8-bit unsigned integers, so
        we need to combine the two received values by concatenation.
        We then simply cast it as a 16-bit int.
    */
        
        
    I2C_MASTER_1_MasterSendStart(0x68, 0);      //start the transaction
    I2C_MASTER_1_MasterWriteByte(0x3B);         //The register address for the X-acceleration is 0x3B
    I2C_MASTER_1_MasterSendRestart(0x68, 1);    //MasterSendRestart sends another start condition without needing to send a stop condition first
    
    xtopByte = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);       //receive the first byte and acknowledge to signal we are not done reading
    xbottomByte = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);    //receive the second byte and non-acknowledge to signal we are done reading

    ytopByte = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_ACK_DATA);       //receive the third byte and acknowledge to signal we are not done reading
    ybottomByte = I2C_MASTER_1_MasterReadByte(I2C_MASTER_1_NAK_DATA);    //receive the fourth byte and non-acknowledge to signal we are done reading
    
    I2C_MASTER_1_MasterSendStop();      //end the transaction
    
    xcombinedData = ((uint16_t)xtopByte << 8) | (uint16_t)xbottomByte;     //combine the two bytes into one 16-bit unsigned integer (left shift the upper byte by 8 and then OR it with the lower byte)
    ycombinedData = ((uint16_t)ytopByte << 8) | (uint16_t)ybottomByte; 
    
    xgeeForce = (int16_t)xcombinedData / LSBperGee; //interpret uint16 as int16 (which it already is). dividing by a float casts to float automatically
    ygeeForce = (int16_t)ycombinedData / LSBperGee;

    if ((xgeeForce > .25) && (xgeeForce > ygeeForce)) LED_MODE = 1;         //if xgee is higher than threshold and higher than ygee, solid LED
    else if ((ygeeForce > .25) && (ygeeForce > xgeeForce)) LED_MODE = 2;    //if ygee is higher than threshold and higher than ygee, blinking LED
    else LED_MODE = 0;                                                      //otherwise, LED is off
    
    runLED();
    
    }
}

void runLED(){
    switch (LED_MODE){
        case 0:             //mode 0: LED is off
            LED_Write(0);
            break;
            
        case 1:             //mode 1: LED is solidly on
            LED_Write(1);
            break;
            
        case 2:             //mode 2: LED is blinking
            LED_Write(1);
            CyDelay(100);
            LED_Write(0);
            CyDelay(100);
            break;
    }
}

void MPU_ByteWrite(uint8_t reg, uint8_t byte){
    /* Writes a single Byte to the I2C device at address 0x68.
    parameters:
        uint8_t reg: the register on the device to write to
        uint8_t byte: the byte to be written
    
    returns:
        void
    */
    
    I2C_MASTER_1_MasterSendStart(0x68, 0);      //start a write to the MPU to begin the transaction
    I2C_MASTER_1_MasterWriteByte(reg);          //send the address of the register we want to write to
    I2C_MASTER_1_MasterWriteByte(byte);         //send the byte
    I2C_MASTER_1_MasterSendStop();              //send the stop signal to end the transaction
}

    
/* [] END OF FILE */
